<?php
/*************************************************************************
 *    @file AprivaGW.php                                                  
 *    @brief The following class wrapps all necessary communication to    
 *           the Apriva Gateway via our XML interface.  The XML interface
 *           is SSL with Client Authentication.                           
 *    @author P Heit                                                      
 *    @date Sept 21, 2013                                                 
 *                                                                        
 *    Copyright (c) 2013 by                                               
 *       Apriva LLC                                                       
 *    All rights reserved.                                                
 *                                                                        
 *    This software is furnished under a license and may be used and      
 *    copied only in accordance with the terms of such license and with   
 *    the inclusion of the above copyright notice. This software or       
 *    any other copies thereof may not be provided or otherwise made      
 *    available to any other person or entity. Neither title to nor       
 *    ownership of the software is hereby transferred.                    
 *                                                                        
 *************************************************************************/
     class AprivaGW {
     		
     	protected $socket = NULL;
      	protected $streamContent;
      	protected $timeout;
      	protected $ip;
      	protected $port;

      	protected $debugOn;
    	
	   	// Constructor setup some of the variables to be used.   
    	function __Construct($ip, $port, $timeout)
    	{
    		$this->ip = $ip;
    		$this->port = $port;
    		
    		$this->debugOn = false;

			// Non blocking sockets used so set the timeout desired.
   			$this->timeout = $timeout;
    	}
    	
    	// Step #1. Setup the client certificate in the stream context    
    	function ConfigureClientCredentials($pem_filename)
    	{
			$this->streamContent = stream_context_create();
 			$pem_file = dirname(__FILE__) . '/' . $pem_filename;   // Assumes the cert is in the same directory can be updated.
			$this->Trace( "Loading Cert from ($pem_file)");

 			try {
 				return stream_context_set_option($this->streamContent, 'ssl', 'local_cert', $pem_file);
			} catch (Exception $e) {
    			$this->Trace( 'Caught exception (In ConfigureClientCredentials): ',  $e->getMessage(), "\n");
			}
			return false;
	   	}

    	// Step #2. Trust the server connection.  PEM file information only need if not a trusted Regsitrar such as Verisign.    
    	function ConfigureServerTrust($trustHost, $capem_filename = null)
    	{
 			$capem_file = dirname(__FILE__) . '/' . $capem_filename;  // Assumes the cert is in the same directory can be updated.
			$this->Trace( "Loading CA Cert from ($capem_file)" );
   	
 			try {
				if (!stream_context_set_option($this->streamContent, 'ssl', 'verify_host', $trustHost))
					return false;
    			return stream_context_set_option($this->streamContent, 'ssl', 'cafile', $capem_file);
			} catch (Exception $e) {
    			$this->Trace( 'Caught exception (In ConfigureServerTrust): ',  $e->getMessage(), "\n");
			}
			
			return false;
	   	}

   		// Step #3. Send and receive a request 
     	function ProcessRequest ($request, &$paymentResp, $isReversal = false)
    	{
			if (!$this->Connect())
				return false;
				
			try {
		   		// Send the request to the Gateway
				$this->Trace("Sending Request\n");
    			$this->Trace($request->asXML());

   				if (!fwrite($this->socket, $request->asXML()))
   				{
					$this->Trace ("Sending failed to communicate with the Apriva GW");
					CleanupSocket();
					return false;
				}

	    		$this->Trace("Reading Response (Listen Timeout for max $this->timeout sec)\n");
				
				// Start a timer so we are not stuck waiting for the host.
	 			$start = microtime(true);

   				while (!feof($this->socket) && (microtime(true) - $start) < $this->timeout)    				
   				{
        			$response = fgets($this->socket, 1024);
	  				if (strlen($response) > 0)
	   		 		{
		  				$this->Trace("$response\n");   		 		
 	   			    	$paymentResp =  new AprivaPaymentResp($response);
						$this->CleanupSocket();
				    	return true;
					}
   		 		}
 
 				// No response received.  Either the socket went down or we hit our time limit.
				$this->Trace ("No Response Received\n");
		    			    			    	
 			} catch (Exception $e) {
    			$this->Trace( 'Caught exception (In ProcessRequest): ',  $e->getMessage(), "\n");
			}

			$this->CleanupSocket();

			// Don't reverse a reversal
			if (!$isReversal)
			{
				// So we know we sent the request but we didnt get a response.  We need to 
				// reverse the transaction on the host to make sure it didnt happen to process.
				try {
					$this->SendReversal($request->GetSystemTraceNumber(), $request->GetMerchantNumber());
  				} catch (Exception $e) {
    				$this->Trace( 'Caught exception (In ProcessingReversal): ',  $e->getMessage(), "\n");
				}
			}

			return false;
    	}
    	    
   		// Establish the gateway connection. 
     	function Connect ()
    	{
 			// Just in case it was called already
 			if ($this->socket != NULL)
 				return true;
 				
 			try {
	 			// Create a stream socket on IP:Port
 				$this->socket = stream_socket_client("ssl://{$this->ip}:{$this->port}", $errno, $errstr, 30, STREAM_CLIENT_CONNECT, $this->streamContent);
				if (!$this->socket) 
				{
					$this->Trace( "Couldn't Connect to Apriva ...$errstr ($errno)");
					return false;
				}

				// Set non blocking so we can timeout.  This is optional.
	 			socket_set_blocking($this->socket, false);

				$this->Trace( "\n\nConnected");
				return true;
 			} catch (Exception $e) {
    			$this->Trace( 'Caught exception (In Connect): ',  $e->getMessage(), "\n");
			}

   			return false;
		}

     	// Cleanup the Socket.  Assumes the GW did the closing. 
	   	function CleanupSocket ()
    	{
			unset($this->socket);
			$this->socket = NULL;
		}

     	// Build and send a reversal request. 
	   	function SendReversal($stan, $merch)
    	{
			$this->Trace ("Sending Reversal\n");
			
			// Build a reversal / cancel request of the for the submitted redit sale
			$revObj = new AprivaReversalReq();
			$revObj->SetSystemTraceNumber($stan);
			$revObj->SetMerchantNumber($merch);
		
			// Connect and send to Gateway
			if (!$this->ProcessRequest($revObj, $resp, true))
				throw new Exception('Timeout transaction failed to reverse.');
			
			// The Gateway is processing something so our request was rejected.
			if ($resp->GetResultCode() == "59" && $resp->GetResultTxt() == "Trans Processing")
				throw new Exception('Transaction is blocked processing.  reversal attempt failed.');				
    	}    
    	    	
   		// Optional. Turn on an off Debugging 
        function SetTracing($enable)
    	{
    		$this->debugOn = $enable;
    	}

     	// Trace details to the screen or a file if desired. 
        function Trace($msg)
    	{
    		if ($this->debugOn)
    			echo $msg, "\n";
    	}
   
    }
    
   ?>
